use wasm_bindgen::prelude::*;
use dist_text::crdts::list::Op;
use dist_text::text::Text;

#[wasm_bindgen(getter_with_clone)]
pub struct State {
    pub text: String,

    #[wasm_bindgen(skip)]
    pub inner: Text,
}

#[wasm_bindgen]
impl State {
    // wasm does not support trait impls yet, hence the default implementation here
    pub fn default() -> Self {
        Self { text: String::default(), inner: Text::default() }
    }

    pub fn apply(&mut self, ops: JsValue) -> String {
        let ops: Vec<Op<u16, String>> = serde_wasm_bindgen::from_value(ops).unwrap_or(Vec::new());
        self.inner.apply_ops(ops);

        self.inner.to_string()
    }

    pub fn execute(&mut self, action: &str, start: usize, end: usize, data: &str, src: &str) -> JsValue {
        let ops = match action {
            "insertText" | "insertFromPaste" => self.inner.insert(start, end, data, src),
            "insertLineBreak" => self.inner.insert_linebreak(start, end, src),
            "deleteContentBackward" => self.inner.delete_backward(start, end, src),
            "deleteContentForward" => self.inner.delete_forward(start, end, src),
            &_ => Vec::default()
        };

        serde_wasm_bindgen::to_value(&ops).unwrap_or_default()
    }
}
